import os
from dataclasses import dataclass

import gpiod


@dataclass
class DInAttr:
    chip: int
    line: int
    gpiod_line: type(gpiod.libgpiodcxx.line)
    edge: bool

@dataclass
class DOutAttr:
    chip: int
    line: int
    gpiod_line: type(gpiod.libgpiodcxx.line)

class DIO_Mgr:
    DI1_PORT = 1
    DI2_PORT = 2
    DO1_PORT = 1
    DO2_PORT = 2

    def __init__(self, din1, din2, dout1, dout2):
        self._dins = dict()
        self._dins['1'] = DInAttr(din1[0], din1[1], None, False)
        self._dins['2'] = DInAttr(din2[0], din2[1], None, False)
        self._douts = dict()
        self._douts['1'] = DOutAttr(dout1[0], dout1[1], None)
        self._douts['2'] = DOutAttr(dout2[0], dout2[1], None)

        self.set_io_port()

    def set_io_port(self):
        try:
            config = gpiod.line_request()
            config.consumer = "DIO_Mgr"
            config.request_type = gpiod.line_request.DIRECTION_INPUT
            DIO_Mgr._setup_port(self._dins, config, '1')
            DIO_Mgr._setup_port(self._dins, config, '2')
            config = gpiod.line_request()
            config.consumer = "DIO_Mgr"
            config.request_type = gpiod.line_request.DIRECTION_OUTPUT
            DIO_Mgr._setup_port(self._douts, config, '1')
            DIO_Mgr._setup_port(self._douts, config, '2')
        except:
            print("DIO_Mgr: set_io_port() is Error!")

    def get_DI1(self):
        return self._dins['1'].gpiod_line.get_value()

    def get_DI2(self):
        return self._dins['2'].gpiod_line.get_value()

    def get_DI(self, port):
        return self._dins[str(port)].gpiod_line.get_value()

    def set_DO1(self, isHigh):
        return self._douts['1'].gpiod_line.set_value(1 if isHigh else 0)

    def set_DO2(self, isHigh):
        return self._douts['2'].gpiod_line.set_value(1 if isHigh else 0)

    def set_DO_Output(self, port, isHigh):
        return self._douts[str(port)].gpiod_line.set_value(1 if isHigh else 0)

    def get_DI_edge(self, port):
        return self._dins[str(port)].edge

    def set_DI_edge(self, port, val):
        self._dins[str(port)].edge = val

    @staticmethod
    def _setup_port(port_dic, config, port):
        dio_attrs = port_dic[port]
        dio_attrs.gpiod_line = gpiod.chip(dio_attrs.chip).get_line(dio_attrs.line)
        dio_attrs.gpiod_line.request(config)

#
# EOF
#
